@@ -9,11 +9,10 @@
 #include <unistd.h>
 #include <errno.h>
 #include <assert.h>
+#include <libaio.h>
 
 #include "../fio.h"
 
-#ifdef FIO_HAVE_LIBAIO
-
 struct libaio_data {
 	io_context_t aio_ctx;
 	struct io_event *aio_events;
@@ -304,27 +303,6 @@ static struct ioengine_ops ioengine = {
 	.option_struct_size	= sizeof(struct libaio_options),
 };
 
-#else /* FIO_HAVE_LIBAIO */
-
-/*
- * When we have a proper configure system in place, we simply wont build
- * and install this io engine. For now install a crippled version that
- * just complains and fails to load.
- */
-static int fio_libaio_init(struct thread_data fio_unused *td)
-{
-	log_err("fio: libaio not available\n");
-	return 1;
-}
-
-static struct ioengine_ops ioengine = {
-	.name		= "libaio",
-	.version	= FIO_IOOPS_VERSION,
-	.init		= fio_libaio_init,
-};
-
-#endif
-
 static void fio_init fio_libaio_register(void)
 {
 	register_ioengine(&ioengine);
