@@ -26,18 +26,10 @@
 #include <time.h>
 
 #include "fio.h"
-#include "hash.h"
 #include "smalloc.h"
-#include "verify.h"
-#include "trim.h"
-#include "diskutil.h"
-#include "profile.h"
-#include "lib/rand.h"
-#include "memalign.h"
-#include "server.h"
 
-uintptr_t page_mask;
-uintptr_t page_size;
+uintptr_t page_mask = 0;
+uintptr_t page_size = 0;
 
 static int endian_check(void)
 {
@@ -53,10 +45,10 @@ static int endian_check(void)
 	else if (u.c[0] == 0x12)
 		le = 1;
 
-#if defined(FIO_LITTLE_ENDIAN)
+#if defined(CONFIG_LITTLE_ENDIAN)
 	if (be)
 		return 1;
-#elif defined(FIO_BIG_ENDIAN)
+#elif defined(CONFIG_BIG_ENDIAN)
 	if (le)
 		return 1;
 #else
@@ -79,6 +71,10 @@ int main(int argc, char *argv[], char *envp[])
 		return 1;
 	}
 
+#if !defined(CONFIG_GETTIMEOFDAY) && !defined(CONFIG_CLOCK_GETTIME)
+#error "No available clock source!"
+#endif
+
 	arch_init(envp);
 
 	sinit();
